## Growth Curves R code

## Developed for frog growth curves in Bell, B.D. and Pledger, S.A. (2022):
## "Post-metamorphic body growth and remarkable longevity in two terrestrial
## New Zealand frogs (Leiopelma archeyi and L. hamiltoni)",
## New Zealand Journal of Ecology.

## Detailed version of R code with descriptions and a typical data set.

## The aim is to find the maximum likelihood estimates for two von
## Bertalanffy growth curves, one for each sex, where there is adult
## sexual dimorphism. The data are from a capture-recapture study,
## with many individuals having fewer than four length measurements.
## Each individual has sex and age at first measurement (AFM) unknown.

## This R code uses finite mixtures, with the EM algorithm wrapped
## around a growth curve analysis to provide maximum likelihood estimates
## of the two curves. See the file "readme.pdf" for technical details.
## This code is Version 1, serving as a proof of concept that the
## wrapping will work. Version 2 will change to using Fabens method
## and will include a component of variance due to individual variation
## as well as random measurement variance included here.

## Code from this file may be copied and pasted into R. Anything
## following ## in a line is comment only and will be ignored by R.
## In order to run this code with an example, have the data file
## "demo.csv" in the same folder.

#######################################################
##                Preliminaries
#######################################################

## Open R and call in two R packages:
## ----------------------------------

library(nlme)
library(mvtnorm)

## Define the von Bertalanffy function:
## ------------------------------------

## Maps from x = Age to response y = SVL (snout-vent length).

## Notation "Asym" (asymptote), "resp0" (length at age 0) and "lrc"
## (log of rate constant) from Pinheiro and Bates (2000),
## asymptotic curve. 

vb.fn <- function(x, Asym = Asym.in, resp0 = resp0.in, lrc = lrc.in){
    k <- exp(lrc)
    y = Asym - (Asym - resp0)*exp(-k*x)
    return(y)
}

## Set up the input data:
## ----------------------

## Need data frame with columns ID, Time, SVL.

demo.df <- read.csv("demo.csv", header = TRUE)

head(demo.df)
str(demo.df)

## Note that ID is a character variable. Ensure it becomes a factor:

demo.df$ID <- factor(demo.df$ID)

## The demo dataframe has 231 SVL measurements from 50 frogs.
## It is a random subset of data from Bell's Grid One on Maud Island.
## Dates have been shifted, but the crucial timing between dates
## for each frog has been kept intact.

## Set up working data called new.df:
## ----------------------------------

new.df <- demo.df

## Optionally, modify or check the data at this stage.
## Here, we restrict the data to individuals with at least four SVL
## measurements, in order to use the nlme package for repeated measures.

new.df <- new.df[!is.na(new.df$SVL),]
OK.IDs <- names(table(new.df$ID))[table(new.df$ID) > 3]
new.df <- new.df[new.df$ID %in% OK.IDs,]
new.df$ID <- factor(new.df$ID)
str(new.df)

## This reduces demo data from 50 to 26 individuals.
## Version 2 of this code will switch to Fabens' method in order to
## allow fuller use of the data.

## Plot raw data, SVL by Time:
## ---------------------------

## Lines connect sequences from each individual.
                                 
plot(new.df$Time,new.df$SVL,
    xlab = "Calendar Time (years)",
    ylab = "Length SVL (mm)")
for (i in 1:length(levels(new.df$ID))){
    this.df <- new.df[new.df$ID == levels(new.df$ID)[i],]
    lines(this.df$Time, this.df$SVL, lty = 5)
    }

## Note the possibility of two asymptotes at the top right.
## However, we need "Age" on the x axis rather than calendar time
## in order to fit growth curves. Each frog's chain of measurements
## must be slid left or right, retaining the spacings, to estimate
## their dates of birth and align them at age x=0.

################################################################
##          Set up constants to use in the EM algorithm
################################################################

## Sample size constants:
## ----------------------

nObs <- nrow(new.df)              ## Number of SVL measurements observed
nObs
nID <- length(unique(new.df$ID))  ## Number of frogs
nID
Ni <- table(new.df$ID)            ## Vector of number of obs. per frog

## The demo data has 177 observations spread over 26 frogs.

## Other constants:
## ----------------

## Age M at which the frogs are assumed to be mature (fully grown):
M <- 5
## Assumed fraction of the way through the calendar year when a
## birth pulse occurs:
BP <- 0.125    ## 0.125 is for mid-February
## Number of groups in the finite mixture:
G <- 2*(M+1)

## Names:
## ------

IDnames <- levels(new.df$ID)
Anames <- paste("A",as.character(0:M),sep="")  ## Names for levels of AFM factor
Snames <- paste("S",as.character(1:2),sep="")  ## Names for levels of SEX factor
Gnames <- c(paste(Anames,"S1",sep=""),paste(Anames,"S2",sep=""))
          ## Names for levels of cross-classified groups

## Age list, a list of possible age vectors which will be called upon in
## the algorithm when updating age estimates:

age.list <- vector("list",nID)
names(age.list) <- IDnames
for (i in 1:nID){
    this.df <- new.df[new.df$ID == IDnames[i],]
    this.ID <- IDnames[i]
    J <- nrow(this.df)   ## J = Ni, the no. obs. of SVL for this ID
    y.vect <- this.df$SVL  ## Data, vector of responses, yj, j=1:J.
    t.vect <- this.df$Time ## Also of length J.
    t1 <- t.vect[1]
    ## Construct M+1 possible scenaria:
    age.mat <- matrix(NA,M+1,J) ## Store possible age vectors
    rownames(age.mat) <- Anames
    colnames(age.mat) <- paste("J",1:J,sep="")
    for (a in 1:(M+1)){
        a1 <- a-1  ## Age group, floor(x1), e.. 0-1, 1-2, etc.
	## Two possible years of birth (yob):
        if ((t1 - floor(t1) >= BP)) yob <- floor(t1) - a1
        if ((t1 - floor(t1) < BP)) yob <- floor(t1) - 1 - a1
	## Date of birth, dob, in YM:
	dob <- yob + BP
	## Age at first capture, in YM:
	x1 <- t1 - dob
	## Vector of ages at which SVL was measured
	x.vect <- x1 + t.vect - t1  ## Also of length J.
	age.mat[a,] <- x.vect
    }
    age.list[[i]] <- age.mat
}

## Long data frame with G blocks, one per scenario (AFM by SEX):
## -------------------------------------------------------------

## Start with columns ID, Time, SVL:
short.df <- new.df[,c(1,4,5)]
head(short.df)
nObs <- nrow(short.df)
short.df$Age <- rep(NA,nObs)
## Make G copies for a long data frame:
long.df <- short.df
for (copies in 1:(G-1)) long.df <- rbind(long.df,short.df)
## Put in factor names:
long.df$AFM <- as.factor(rep(c(Anames,Anames),each= nObs))
long.df$SEX <- as.factor(rep(Snames,each = (M+1)*nObs))
long.df$GRP <- as.factor(rep(Gnames,each=nObs))
## Put in indices:
long.df$i <- rep(NA,nrow(long.df))
for (i in 1:nID) long.df$i[long.df$ID == levels(long.df$ID)[i]] <- i
long.df$a <- rep(c(1:(M+1),1:(M+1)),each= nObs)
long.df$s <- rep(c(1,2),each = (M+1)*nObs)
long.df$g <- rep(1:G,each=nObs)
## Put in age vectors to be used in each scenario:
for (i in 1:nID) for (a in 1:(M+1)){
    long.df$Age[(long.df$i == i)&(long.df$a == a)] <- age.list[[i]][a,]
}

dim(long.df)
head(long.df)
str(long.df)

## The long data frame has 12 repeats of the data set, each repeat assuming
## the data is in one of 12 groups, one per AFM by SEX combination.

## The AFM may be one of the following levels:
levels(long.df$AFM)
## [1] "A0" "A1" "A2" "A3" "A4" "A5"
## Where "A0" represents age 0-1 year at first measurement, "A1" is age 1-2,
## and finally "A5" represents age 5+ (fully grown) at first measurement.
## The levels of Sex are assumed to be "S1" for the sex with higher asymptote
## and "S2" for the lower asymptote.

## Plot possible data chains:
## --------------------------

## In a plot of SVL versus Age, each frog has six possibilities for its
## vector of Age values, so the following plot has six parallel trajectories
## for each frog.

maxage <- max(long.df$Age)
maxSVL <- max(long.df$SVL)

plot(long.df$Age, long.df$SVL, type = "n",
     xlim = c(0,maxage+1), xlab = "Age (years)",
     ylim = c(0,maxSVL+1), ylab = "SVL (mm.)",
     main = "SVL by possible Age data chains")
for (i in 1:nID){
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="grey90",pch = 16,cex=0.5)
    	lines(thisig.df$Age,thisig.df$SVL,col="grey90",lty=5,lwd=0.5)
    }
}

## Note six alternative age vectors for each frog.

## Highlight a few chosen IDs:
i <- 1
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="turquoise3",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="turquoise3")
    }
i <- 2
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="orange1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="orange1")
    }
i <- 9
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="palevioletred1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="palevioletred1")
    }
i <- 16
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="purple1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="purple1")
    }
legend(20,20,
    c("Frog.01","Frog.02","Frog.16","Frog.28"),
    lwd = c(1,1,1,1),pch = c(16,16,16,16),
    col = c("turquoise3","orange1","palevioletred1","purple1"))

## Note the suggestion of separation into two asymptotes for mature frogs.
## We will try starting the algorithm with asymptotes around 42 and 37.
## Note also occasional decreases in SVL for some individuals. This
## will be attributed to random measurement error.

## Each of these four frogs has a set of six possible trajectories, one
## for each possible AFM value.
## Note that Frog 1 (turquoise) has a level trajectory, indicating it
## is already grown when first caught. The EM algorithm will allocate
## it with very high probability to the "A5" and "S2" cluster. Only the
## furthest right of its six possible traces will be used in the EM model
## fitting. Similarly frogs 2 and 9 (orange and pink) will have "A3"
## or "A4" selected. In this way there will ultimately be very little
## evidence for points at the top left of the plot, as seen later.
## However, Frog 28 (purple) was very small when first caught and is almost
## certainly in its first year when first caught. It will have "A0" chosen
## during the EM algorithm, choosing the left hand one of its alternative
## traces. It will be a strong contributor to the low part of the growth
## curves. This selection of the most likely AFM simply imitates and
## automates the methods used by Bell in previous publications. It sets up
## the extension to probabilistic clustering, enabling us to maximise the
## use of all available information rather than discarding individuals
## which don't have a very high degree of certainty of allocation.


###################################################################
##              Run the EM Algorithm
###################################################################

#############################
##     Step 0: Initialise
#############################

## Initialise parameter estimates, to be updated during EM.
## --------------------------------------------------------

## Two vonB curves, one per sex:
## -----------------------------

vb2.pars <- matrix(c(42,6,-0.5,37,6,-0.5),3,2)
dimnames(vb2.pars) <- list(c("Asym","resp0","lrc"),c("Sex1","Sex2"))
vb2.pars

## The parameter names are those used in R for this curve.
## "resp0" is the response (SVL) at age 0, i.e. length at "birth".
## "lrc" is the log of the instantaneous growth constant.

## Proportions of individuals in each scenario (group) g:
## ------------------------------------------------------

pi.vect <- rep(1/G, G)   ## Uninformative start is OK.
names(pi.vect) <- Gnames

## Set up matrix to store group membership updates:
## -----------------------------------------------

## Z.mat is matrix of posterior probs. individual i in group g:
Z.mat <- matrix(NA,nID,G)
dimnames(Z.mat) <- list(IDnames,Gnames)
Z.mat

## Set up array to store individual posterior probability updates:
## ---------------------------------------------------------------

pp.array <- array(NA, c(nID, M+1, 2))
dimnames(pp.array) <- list(IDnames,Anames,Snames)
pp.array

## Set up looping for EM:
## ----------------------

maxcycles <- 20   ## Set an upper limit in case of non-convergence
this.cycle <- 0

## Set up matrix to store the sequence of EM fits:
## -----------------------------------------------

vBparests.mat <- matrix(rep(NA,6),1,6)
colnames(vBparests.mat) <- c("Sex1.Asym","Sex1.resp0","Sex1.lrc",
                          "Sex2.Asym","Sex2.resp0","Sex2.lrc")


################################################ 
##     Steps 2 and 3: E-step and M-step
################################################

## Alternate the E (expectation) and M (maximisation) steps until happy.

## Copy and paste everything up to ## End of EM cycles

while(this.cycle < maxcycles){
    this.cycle <- this.cycle + 1
    print(paste("Iter",this.cycle))

    ###################################################
    ##    E step - update posterior probabilities
    ###################################################

    ## Evaluate and store current fits to the two current curves:
    long.df$fitted <- rep(NA,nrow(long.df))
    for (s in 1:2){
        Asym.in <- vb2.pars[1,s]
	resp0.in <- vb2.pars[2,s]
	lrc.in <- vb2.pars[3,s]
	long.df$fitted[long.df$s == s] <- vb.fn(long.df$Age[long.df$s == s])
    }

    ## For each ID, find vector of E-step numerators then normalise.
    for (i in 1:nID){
        thisi.df <- long.df[long.df$i == i,]
	Ji <- nrow(thisi.df)/G      ## No. SVL measurements of ID i
	num.vect <- rep(NA,G)
	for (g in 1:G){
	    thisig.df <- thisi.df[thisi.df$g == g,]
	    x.vect <- thisig.df$Age         ## Possible age vector
	    y.vect <- thisig.df$SVL         ## Observed SVLs
	    Ey.vect <- thisig.df$fitted     ## Expected SVLs
	    ## Use MSE to estimate common var of raw resids:
	    var.hat <- mean((y.vect-Ey.vect)^2)
	    VCOV.mat <- diag(rep(var.hat,Ji))
	    ## Get pi times density function of multivariate normal:
	    num.vect[g] <- pi.vect[g] * dmvnorm(y.vect, mean = Ey.vect,
   	                                        sigma = VCOV.mat)
	}
	## Normalise to sum to 1, i.e. get posterior probs:
	denom <- sum(num.vect)
	Z.mat[i,] <- num.vect/denom
    }

    ## Row sums should be 1.
    apply(Z.mat,1,sum)


    ##########################################################
    ##      M step - update pi and vB parameter estimates
    ##########################################################

    ## Update prior prob estimates:
    ## ----------------------------

    pi.vect <- apply(Z.mat,2,mean)

    ## Generate pseudo-data for generalised EM (GEM) algorithm:
    ## --------------------------------------------------------

    ## Assign each ID to its currently best AFM and hence build
    ## its best current estimated age vector.
    ## Use y = SVL versus x = current estimated Age as pseudo-data
    ## and fit the two curves.

    pseudo.df <- new.df
    pseudo.df$Age <- rep(NA,nrow(pseudo.df))
    pseudo.df$prob1 <- rep(NA,nrow(pseudo.df))   ## Current prob sex 1

    for (i in 1:nID){
        this.ID <- rownames(Z.mat)[i]
        pp.mat <- matrix(Z.mat[i,],M+1,2)
        dimnames(pp.mat) <- list(Anames,Snames)
        pp.array[i,,] <- pp.mat
        AFM.prob <- apply(pp.mat,1,sum)   ## Marginal AFM     
        best.a <- (1:(M+1))[AFM.prob == max(AFM.prob)]
        best.agevect <- age.list[[i]][best.a,]
        pseudo.df$Age[pseudo.df$ID == this.ID] <- best.agevect
        prob1 <- pp.mat[best.a,1]/sum(pp.mat[best.a,])
        pseudo.df$prob1[pseudo.df$ID == this.ID] <- prob1
    }

    pseudo.df$prob2 <- 1 - pseudo.df$prob1

    ## Update the two fitted curves and the vB parameter estimates:
    ## ------------------------------------------------------------

    sex1.nls <- nls(SVL ~ SSasymp(Age, Asym, resp0, lrc),
                    data = pseudo.df, weights = prob1)

    sex2.nls <- nls(SVL ~ SSasymp(Age, Asym, resp0, lrc),
                    data = pseudo.df, weights = prob2)

    vb2.pars[,1] <- summary(sex1.nls)$coef[,1]
    vb2.pars[,2] <- summary(sex2.nls)$coef[,1]
    vBparests.mat <- rbind(vBparests.mat, c(vb2.pars))

    print("Updated pars:")
    print(vb2.pars)
}

## End of EM cycles

######################################
##         Results
######################################

## Plot the fitted vB curves:
## --------------------------

vBparests.mat <- vBparests.mat[-1,]


## Plot changing vB ests over the cycles:
## --------------------------------------

plot(1:nrow(vBparests.mat),vBparests.mat[,1],col="red",pch=16,
     ylim = range(vBparests.mat[,c(1,4)]),
     xlab = "EM Cycle",ylab = "Length (mm)",main = "Estimated Asymptotes")
lines(1:nrow(vBparests.mat),vBparests.mat[,1],col="red")
points(1:nrow(vBparests.mat),vBparests.mat[,4],col="blue",pch=16)
lines(1:nrow(vBparests.mat),vBparests.mat[,4],col="blue")
legend(maxcycles-5,mean(vBparests.mat[maxcycles,c(1,4)]),
       c("Sex1","Sex2"),pch = c(16,16),
       lty = c(1,1), col = c("red","blue"))
## Note fast settling to stable estimates

plot(1:nrow(vBparests.mat),vBparests.mat[,2],col="red",pch=16,
     ylim = range(vBparests.mat[,c(2,5)]),
     xlab = "EM Cycle",ylab = "Length (mm)",main = "Estimated resp0")
lines(1:nrow(vBparests.mat),vBparests.mat[,2],col="red")
points(1:nrow(vBparests.mat),vBparests.mat[,5],col="blue")
lines(1:nrow(vBparests.mat),vBparests.mat[,5],col="blue")
legend(maxcycles-5,mean(vBparests.mat[maxcycles,c(2,5)]),
       c("Sex1","Sex2"),pch = c(16,16),
       lty = c(1,1), col = c("red","blue"))
## A bit erratic at the start, but soon sorts itself out

plot(1:nrow(vBparests.mat),exp(vBparests.mat[,3]),col="red",pch=16,
     ylim = range(exp(vBparests.mat[,c(3,6)])),
     xlab = "EM Cycle",ylab = "Length (mm)",main = "Estimated rate constant k")
lines(1:nrow(vBparests.mat),exp(vBparests.mat[,3]),col="red")
points(1:nrow(vBparests.mat),exp(vBparests.mat[,6]),col="blue",pch=16)
lines(1:nrow(vBparests.mat),exp(vBparests.mat[,6]),col="blue")
legend(maxcycles-5,mean(vBparests.mat[maxcycles,c(3,6)]),
       c("Sex1","Sex2"),pch = c(16,16),
       lty = c(1,1), col = c("red","blue"))
## Have plotted rate constant k (positive) although calculations worked with
## log(rate constant), lrc.

## Note that only about 10-12 EM cycles were needed to achieved a rebalancing
## of the correlated estimates and convergence to the final parameter estimates.


## Check that all fitted values are on two curves:
## -----------------------------------------------

plot(long.df$Age, long.df$fitted)


## Plot final vB curves over the possible data chains:
## ---------------------------------------------------

## Possible data chains, as before:

maxage <- max(long.df$Age)
maxSVL <- max(long.df$SVL)

plot(long.df$Age, long.df$SVL, type = "n",
     xlim = c(0,maxage+1), xlab = "Age (years)",
     ylim = c(0,maxSVL+1), ylab = "SVL (mm.)",
     main = "Final vB curves and SVL by possible Age data chains")
for (i in 1:nID){
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="grey90",pch = 16,cex=0.5)
    	lines(thisig.df$Age,thisig.df$SVL,col="grey90",lty=5,lwd=0.5)
    }
}

## Highlight the same few chosen IDs, as before:
i <- 1
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="turquoise3",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="turquoise3")
    }
i <- 2
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="orange1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="orange1")
    }
i <- 9
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="palevioletred1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="palevioletred1")
    }
i <- 16
    for (g in 1:G){
        thisig.df <- long.df[(long.df$i == i)&(long.df$g == g),]
    	points(thisig.df$Age,thisig.df$SVL,col="purple1",pch = 16)
    	lines(thisig.df$Age,thisig.df$SVL,col="purple1")
    }
legend(20,20,
    c("Frog.01","Frog.02","Frog.16","Frog.28"),
    lwd = c(1,1,1,1),pch = c(16,16,16,16),
    col = c("turquoise3","orange1","palevioletred1","purple1"))

## Now add the fitted curves, red for higher asymptote (females),
## blue for lower (males):

Asym.in <- vb2.pars[1,1]
resp0.in <- vb2.pars[2,1]
lrc.in <- vb2.pars[3,1]
curve(vb.fn, from = 0, to = maxage+1,
      col = "red", lwd = 2, add = T)
Asym.in <- vb2.pars[1,2]
resp0.in <- vb2.pars[2,2]
lrc.in <- vb2.pars[3,2]
curve(vb.fn, from = 0, to = maxage+1,
      col = "blue", lwd = 2, add=T)
legend(20,9,c("vB curve 1","vB curve 2"),
       lwd = c(2,2),col = c("red","blue"))

## The above plot is far too messy. Instead:

## Plot pseudo data and fitted curves:
## -----------------------------------

## This method was used for the figures in Bell and Pledger (2022).

plot(pseudo.df$Age, pseudo.df$SVL, pch = 16, col = "grey",
     xlab = "Age (years)", ylab = "SVL (mm)",
     ylim = c(5,46))
title(main = "Fitted vB curves and pseudo data")
mysubtitle = "Age uses best choice of AFM for each frog"
mtext(side = 3, line = 0.25, at = -22.5, adj = -2, mysubtitle)
for (i in 1:nID){
    thisi.df <- pseudo.df[pseudo.df$ID == levels(pseudo.df$ID)[i],]
    lines(thisi.df$Age,thisi.df$SVL,col="grey",lty=5)
}
## Highlight four frogs:
i <- 1
thisi.df <- pseudo.df[pseudo.df$ID == levels(pseudo.df$ID)[i],]
points(thisi.df$Age,thisi.df$SVL,col="turquoise3",pch = 16,cex=1.3)
lines(thisi.df$Age,thisi.df$SVL,col="turquoise3",lty=1,lwd=1.5)
i <- 2
thisi.df <- pseudo.df[pseudo.df$ID == levels(pseudo.df$ID)[i],]
points(thisi.df$Age,thisi.df$SVL,col="orange1",pch = 16,cex=1.3)
lines(thisi.df$Age,thisi.df$SVL,col="orange1",lty=1,lwd=1.5)
i <- 9
thisi.df <- pseudo.df[pseudo.df$ID == levels(pseudo.df$ID)[i],]
points(thisi.df$Age,thisi.df$SVL,col="palevioletred1",pch = 16,cex=1.3)
lines(thisi.df$Age,thisi.df$SVL,col="palevioletred1",lty=1,lwd=1.5)
i <- 16
thisi.df <- pseudo.df[pseudo.df$ID == levels(pseudo.df$ID)[i],]
points(thisi.df$Age,thisi.df$SVL,col="purple1",pch = 16,cex=1.3)
lines(thisi.df$Age,thisi.df$SVL,col="purple1",lty=1,lwd=1.5)
legend(17,20,
    c("Frog.01","Frog.02","Frog.16","Frog.28","Others"),
    lwd = c(1,1,1,1,1),pch = c(16,16,16,16,16),
    col = c("turquoise3","orange1","palevioletred1","purple1","grey"))
Asym.in <- vb2.pars[1,1]
resp0.in <- vb2.pars[2,1]
lrc.in <- vb2.pars[3,1]
curve(vb.fn, from = 0, to = maxage+1,
      col = "red", lwd = 2, add = T)
Asym.in <- vb2.pars[1,2]
resp0.in <- vb2.pars[2,2]
lrc.in <- vb2.pars[3,2]
curve(vb.fn, from = 0, to = maxage+1,
      col = "blue", lwd = 2, add=T)
legend(17,9,c("vB curve Sex 1","vB curve Sex 2"),
       lwd = c(2,2),col = c("red","blue"))
text(8,27,"Frog 21", col="grey")

## Frogs 2 and 16 are almost certainly female while Frogs 1 and 28
## are almost certainly male. The probabilities for each frog's allocation
## to Sex 1 or Sex 2 can be found from the EM posterior probabilities.

## Note that Frog 21 was caught while small in 1996 and 1997, then much
## later as an adult in 2016 and 2017. This gives rise to a misleading
## diagonal in the plot when joining points with a linear interpolation.

## Allocations of individuals to groups:
## -------------------------------------

## The final estimated probabilities of group membership are found
## in the 3D pp.array, rows = Frogs, columns = AFM,  layers = SEX.

round(pp.array,3)

## Example: Frog.10 is in 5th row of pp.array:
round(pp.array[5,,],3)
## Almost certainly a female (S1), and almost certainly already adult
## when first caught.
## Matches actual data on Frog.10:
new.df[new.df$ID == "Frog.10",]
##Has four readings, all near the higher asymptote.

## Caution: current code tends to assign some as male when they are females
## on their way past the lower asymptote at their final capture.
## Version 2 of this code will define three sexes, including "unknown",
## and only assign it to male if it has stabilised at the lower asymptote.
